@@ -1,4 +1,4 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # -*- coding: utf-8 -*-
 #
 #  hot-backup.py: perform a "hot" backup of a Subversion repository
@@ -29,13 +29,14 @@
 # ====================================================================
 
 # $HeadURL: https://svn.apache.org/repos/asf/subversion/branches/1.14.x/tools/backup/hot-backup.py.in $
-# $LastChangedDate: 2015-12-21 15:02:31 +0000 (Mon, 21 Dec 2015) $
-# $LastChangedBy: danielsh $
-# $LastChangedRevision: 1721179 $
+# $LastChangedDate: 2020-11-14 04:00:12 +0000 (Sat, 14 Nov 2020) $
+# $LastChangedBy: svn-role $
+# $LastChangedRevision: 1883406 $
 
 ######################################################################
 
 import sys, os, getopt, stat, re, time, shutil, subprocess
+import functools
 
 ######################################################################
 # Global Settings
@@ -193,7 +194,7 @@ def comparator(a, b):
     if not inca:
       return -1
     elif not incb:
-      return 1;
+      return 1
     elif (int(inca) < int(incb)):
       return -1
     else:
@@ -219,7 +220,7 @@ def get_youngest_revision():
     raise Exception("Unable to find the youngest revision for repository '%s'"
                     ": %s" % (repo_dir, stderr_lines[0].rstrip()))
 
-  return stdout_lines[0].strip()
+  return stdout_lines[0].strip().decode()
 
 ######################################################################
 # Main
@@ -255,7 +256,7 @@ regexp = re.compile("^" + re.escape(repo) + "-" + re.escape(youngest) +
 directory_list = os.listdir(backup_dir)
 young_list = [x for x in directory_list if regexp.search(x)]
 if young_list:
-  young_list.sort(comparator)
+  young_list.sort(key = functools.cmp_to_key(comparator))
   increment = regexp.search(young_list.pop()).groupdict()['increment']
   if increment:
     backup_subdir = os.path.join(backup_dir, repo + "-" + youngest + "-"
@@ -348,7 +349,7 @@ if num_backups > 0:
   regexp = re.compile("^" + re.escape(repo) + "-[0-9]+(-[0-9]+)?" + ext_re + "$")
   directory_list = os.listdir(backup_dir)
   old_list = [x for x in directory_list if regexp.search(x)]
-  old_list.sort(comparator)
+  old_list.sort(key = functools.cmp_to_key(comparator))
   del old_list[max(0,len(old_list)-num_backups):]
   for item in old_list:
     old_backup_item = os.path.join(backup_dir, item)
